const config = require('./config');

let { databases } = config

const database = {}

database.createDatabase = function (options) {
    let { 
        dbName, 
        timestampData, 
        autoload, 
        done, 
        onload, 
        afterSerialization, 
        beforeDeserialization, 
        corruptAlertThreshold,
        compareStrings  } = options

    this.name = dbName
    this.timestampData = timestampData
    this.autoload = autoload
    this.done = done
    this.onload = onload
    this.afterSerialization = afterSerialization
    this.beforeDeserialization = beforeDeserialization
    this.corruptAlertThreshold  = corruptAlertThreshold 
    this.compareStrings = compareStrings 

    config.createDatabase.apply(this)
}

/**
 * Select database.
 */
database.select = function (schema) { 
    /**
    * @param {string} schema - Schema é a coleção à qual você se refere. Já que a biblioteca não usa referências em variáveis, utilizamos esse meio para referenciar um database e alterar ele sem disponibilizar uma variável global no sistema.
    */
    this.schema = schema
    return this
}


database.find = function (query, options, projection) {
    let { limit , skip ,  sort } = { ...options }
    
    return new Promise((resolve, reject) => {
        databases[this.schema].find(query, projection)
            .sort(sort)
            .limit(limit)
            .skip(skip)
            .exec((err, docs) => {
                if(err) reject(new Error('Something went wrong', err))
                
                resolve(docs)
            })
    })
}

database.insert = function (doc) {
    return new Promise((resolve, reject) => {
        databases[this.schema].insert(doc, (err, newDoc) => {
            if (err) reject(new Error(err))

            resolve(newDoc)
        })
    })
}

database.count = function (query) {
    return new Promise((resolve, reject) => {
        databases[this.schema].count(query, (err, count) => {
            if (err) reject(new Error(err))

            resolve(count)
        })
    })
}

database.remove = function (query, multi, options) {
    return new Promise(async (resolve, reject) => {
        let docs;

        if (options && options.returnDeletedData) {
            docs = await this.select(this.schema).find(query)
        }        

        if (docs && docs.length > 1 && !multi ) {
            throw new Error("You are trying to delete one document, but 'multi' property is set to false")
        }

        databases[this.schema].remove(query, { multi }, (err, numRemoved) => {
            if (err) reject(err)

            resolve({
                numRemoved,
                docs
            })
        })
    })
}

database.flush = function (returnRemoved) {
    return new Promise(async (resolve, reject) => {
        let docs;

        if (returnRemoved) {
            docs = await this.select(this.schema).find()
        }

        databases[this.schema].remove({}, {multi: true}, (err, numRemoved) => {
            if (err) { reject(new Error(err)) }

            resolve({
                numRemoved,
                docs: docs ? docs : null
            })
        })
    })
}

database.update = function (query, updateDoc, options = {}) {
    return new Promise((resolve, reject) => {
        databases[this.schema].update(query, updateDoc, options, (err, numAffected, affectedDocuments, upsert) => {
            if (err) reject(new Error(err))

            resolve({
                numAffected,
                affectedDocuments,
                upsert
            })
        })
    })
}

database.drop = function () {
    return new Promise(async (resolve, reject) => {
        let DBOpenRequest = window.indexedDB.open("NeDB", 4)
        

        DBOpenRequest.onsuccess = function (event) {
            let db = DBOpenRequest.result;
            let timeout = setTimeout( function () { 
                deleteRow.call(database, db)
                clearTimeout(timeout)
                resolve()
            }, 100)
            
        }

        function deleteRow (db) {
            let transaction = db.transaction([ 'nedbdata' ], 'readwrite')
            
            transaction.oncomplete = function (event) { console.log('complete') }
            transaction.onerror = function () { console.log('complete') }
    
            let ObjectStore = transaction.objectStore('nedbdata')
        
            let objectStoreRequest = ObjectStore.delete(this.schema)
        
            objectStoreRequest.onsuccess = function (event) {
                console.log('deleted this database')
            }

        }
    })
}

database.index = function (options) {
    return new Promise((resolve, reject) => {
        if (!options || !options.fieldName) {
            throw new Error('You have to specify a field name to index.')
        }
    
        db.ensureIndex(options, function (err) {
            if (err) reject(new Error(err))

            resolve(`${options.fieldName} has been indexed`)
        })
    })
}

database._nedb = databases

module.exports = database